import javax.swing.*;
import javax.swing.Timer;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Graphics2D;
import java.util.*;

/**
 * Se encarga de manejar la simulacion de PhysicLab
 * Crea el menu Principal
 * Pasa los datos a los graficos
 * 
 */
public class MyWorld implements ActionListener {
	public static int WIDTH = 600;  // in pixels
	public static int HEIGHT = 500; // in pixels
	private double t;              // Simulation time
	private Timer pase;
	private double delta_t;        // in seconds
	private double refreshPeriod;  // in seconds
	/**
	 * Maneja el menu Select
	 */
	protected JMenu menu2;
	/**
	 * Maneja los RadioButtonItem del menu Select
	 */
	protected JRadioButtonMenuItem rbMenuItem;
	/**
	 * Indica que bola(id) ha sido seleccionada
	 */
	private int Selected=-1;
	private Vector2D g;
	private float viscosity;
	private PhysicsLabPanel labPanel;
	private Vector<PhysicsElement> elements;
	ConfigurationMenuListener cListener;
	WorldControlMenuListener wcListener;
	/**
	 * Maneja los graficos de la aplicacion
	 * 
	 */
	private Grafic grafico;
	
	/**
	 * Constructor de MyWorld. En esta version se agrega
	 * los parametros refreshPeriod y deltaTime que son parte
	 * de los requerimientos de la tarea
	 * @param refreshPeriod Tiempo de refresco de la simulacion
	 * @param delta_t	Tiempo delta de la simulacion
	 */
	public MyWorld(double refreshPeriod, double delta_t){
		t = 0;
		this.refreshPeriod = refreshPeriod;      // 80 [ms]
		this.delta_t = delta_t;          // 0.1 [ms]
		pase = new Timer((int)(refreshPeriod*1000), this);
		g = new Vector2D(0, 0);
		viscosity = 0.0f;
		elements = new Vector<PhysicsElement>();
		labPanel = null;
		
	}
	public void setPanel (PhysicsLabPanel p) {
		
		labPanel = p;
		
	}

	public void setDelta_t(double delta) {
		delta_t = delta;
	}
	public void setRefreshPeriod (double rp) {
		refreshPeriod = rp;
		pase.setDelay((int)(refreshPeriod*1000));
	}
	public void setGravity (Vector2D gravity) {
		g = gravity;
	}

	public void setViscosity(float v){
		viscosity = v;
	}
	public void addElement(PhysicsElement e) {
		///JMenuBar setLabMenuBar(cListener,wcListener,this);
		elements.add(e);
		menuSelect();
		labPanel.repaint();
	}


	//elimina el elemento seleccionado.
	public void removeElement(PhysicsElement e) {

		if (e instanceof GSpring){ //'Desengachamos' si es un spring
			GSpring auxSpring=(GSpring)e;
			auxSpring.detachAend();
			auxSpring.detachBend();
		}

		else if (e instanceof GElastic){ //'Desengachamos' si es un elastic
			GElastic auxElastic=(GElastic)e;
			auxElastic.detachAend();
			auxElastic.detachBend();
		}
		else if (e instanceof GFixedHook) //en este caso buscamos el string/elastic asociado
		{
			GFixedHook auxFixed;

			auxFixed=(GFixedHook)e;
			//clonamos el vector de springs para no tener problemas de referencias
			Vector<Spring> v= new Vector<Spring>(auxFixed.getSpringVector());
			for(Spring locale: v)
			{
				//para todos los springs del gancho, se desengachan.
				if(auxFixed.getPosition().equals(locale.getAendPosition()))
				{
					locale.detachAend();
					auxFixed.detachSpring(locale);
				}
				else if(auxFixed.getPosition().equals(locale.getBendPosition()))
				{
					locale.detachBend();
					auxFixed.detachSpring(locale);
				}
			}

		}
		else if (e instanceof GBall)
		{
			GBall auxBall = (GBall)e;
			
				Vector<Spring> v =new Vector<Spring>(auxBall.getSpringVector());
				for(Spring locale: v)
				{
					//para todos los springs de la la, se desengachan.
					if(auxBall.getPosition().equals(locale.getAendPosition()))
					{
						locale.detachAend();
						auxBall.detachSpring(locale);
					}
					else if(auxBall.getPosition().equals(locale.getBendPosition()))
					{
						locale.detachBend();
						auxBall.detachSpring(locale);
					}
				}
		}

		elements.remove(e);
		menuSelect();
		labPanel.repaint();
	}
   
   //para el evento de editar masas y ctes de elasticidad de los elementos
   public void editElement(PhysicsElement e){
	 if (e instanceof GSpring){
		String data = JOptionPane.showInputDialog("Ingrese constante de elasticidad[N/m]");
		 ((GSpring) e).setStiffness(Float.parseFloat(data));
		 }
	else if (e instanceof GBall){
		 String data = JOptionPane.showInputDialog("Ingrese la masa de la la [Kg]");
		 ((GBall) e).setMass(Float.parseFloat(data));
	 }
	else if (e instanceof GElastic){
			String data = JOptionPane.showInputDialog("Ingrese constante de elasticidad[N/m]");
			 ((GElastic) e).setStiffness(Float.parseFloat(data));
			 }
  else JOptionPane.showMessageDialog (labPanel, "Este objeto no puede ser editado");
	 labPanel.repaint();
   }
   
   //Antes de comenzar, setea las velocidades de las las a cero
   public void start() {
	   Ball la;
      if(pase.isRunning()) return;
      for (PhysicsElement e: elements){
         if(e instanceof Ball){
           la =(Ball)e;
           la.setVelocity(new Vector2D(0.0,0.0));
         }
    	  e.setInitialState(g, viscosity);
      }
    	  labPanel.desableMouseListener();
      pase.start();      
   }
   
   //continua con los valores iguales hasta antes de stop()
   public void resume(){
	   if(pase.isRunning()) return;
	      for (PhysicsElement e: elements)
	         e.setInitialState(g, viscosity);
	      labPanel.desableMouseListener();
         
	      pase.start();
	   
   }
   public void stop(){
      pase.stop();
      labPanel.enableMouseListener();
   }
   /**
    * Metodo que simula el paso del tiempo en la simulacion. Aqui se pasan
    * los datos a Graphic grafico para generar las curvas de la grafica.
    */
   public void actionPerformed (ActionEvent event) {  // simulate passing time
      double nextStop=t+refreshPeriod;
      GBall bola ;
      double x = 0 ,y=0;
     for (; t<nextStop; t+=delta_t){
    	 for (PhysicsElement e: elements)
         e.computeNextState(delta_t, g, viscosity);
    	 for (PhysicsElement e: elements)
    	 	{
    		 e.updateState();
    		 //si el elemento del arreglo elements es una bola y es la selecciona
    		 //entonces se graficara
    		 if (e instanceof GBall && Selected== e.getId())
         		{
    			 bola=(GBall)e;
    			 x= bola.getX();
    			 y= bola.getY();
         		}   
    	 	}
 
        	  
          
        }  
     //se pasan los datos a graficar.
      if (x!=0 && y!=0)
      grafico.add(x,y,(float)t,(float)refreshPeriod);
      labPanel.repaint();
   }

   
   
   public void paintComponent(Graphics2D g2d){
      for (PhysicsElement e: elements)
         if (e instanceof Drawable)
            ((Drawable)e).draw(g2d);
   }
   public Drawable find(int x, int y) {
      for (PhysicsElement e: elements)
         if (e instanceof Drawable){
            Drawable de = (Drawable) e;
            if (de.contains(x,y)) return de;
         }
      return null;
   }  

   public AttachableElement findAttachableElement(int x, int y) {
      for (PhysicsElement e :elements)
         if (e instanceof AttachableElement) {
            AttachableElement ae =(AttachableElement) e;
            if (ae instanceof Drawable)
            if (((Drawable)ae).contains(x,y)) return ae;
            
         }
      return null;
   }
   //Verifica si se esta simulando o no
   public boolean isRunning(){
	   if (pase.isRunning())
		   return true;
	   else
		   return false;
   }
/**
 * Crea el menu Select con el tooltip pedido y llama a menuSelect
 * para agregar los RadioButtonItem
 * @return JMenu con el menu Select
 */
   public JMenu CreaMenu(){
	   
	    menu2 = new JMenu("Select");
	    menu2.setToolTipText("Select ball to be plotted.");
	   menuSelect();
	   return menu2;
	     
   }
   
   /**
    * Agrega RadioButtonItems al Menu Select. Esto a medida que se 
    * agregan/eliminar bolas a la simulacion
    */
   public void menuSelect () {  
	      ButtonGroup group=new ButtonGroup();
	 	  menu2.removeAll();  
	      int i;
	     for (i=0; i<elements.size(); i++){
	       if(elements.get(i) instanceof GBall &&
	    		   !(elements.get(i) instanceof GFixedHook)	) {
	    	   int id=-1;
	    	   id=((GBall)elements.get(i)).getId();
	            rbMenuItem = new JRadioButtonMenuItem("Ball"+id,false);
	            
	            
	            ActionListener sel=new ActionListener(){
	               //Cuando se selecciona una nueva bola del menu select
	            	//se llama a select para actualizar la bola a graficar.
	            	public void actionPerformed(ActionEvent event){
	            		
	            		JMenuItem menuItem = (JMenuItem)(event.getSource());
	            		selected(menuItem.getText());


	            		
	            		
	            		
	            	}
	            };
	            rbMenuItem.addActionListener(sel);
	     	    menu2.add(rbMenuItem);
	     	    group.add(rbMenuItem);
	     	} 
	      }  
	    
}
   //se busca en el vector elements la bola que ha sido seleccionada en Select.
   public void selected(String t){
	   //Para obtener el numero de la bola desde el string de Select
	   int x=Integer.parseInt(t.substring(4));
	   //int i;
	   for (int i=0; i<elements.size(); i++){
	       if(elements.get(i) instanceof GBall &&
	    		   !(elements.get(i) instanceof GFixedHook)	   ) {
	    	   GBall ball = (GBall) elements.get(i);
	    	if(ball.getId()==x){
	    		Selected=ball.getId();
	        	break;
	    	}
	    	else 
	    		Selected=-1;
	       }
	   }
	   //se marca la que sera graficada.
	   for (int i=0; i<elements.size(); i++){
	       if(elements.get(i) instanceof GBall &&
	    		   !(elements.get(i) instanceof GFixedHook)	   ) {  
	    	   if(elements.get(i).getId()==Selected){
	    	   ((GBall)elements.get(i)).plot(true);
	    	   grafico.borraGrafic();
	    	   
	    	   }
	    	   else
	    		   ((GBall)elements.get(i)).plot(false);
	       }
	       }
	   labPanel.repaint();
   }
   
   /**
    * Instancia de Graphic en base al panel y el tiempo que sera graficado
    * @param Panel Donde se ubicaran los 3 graficos
    * @param plotWindowTime Tiempo que sera mostrado en los graficos
    */
   public void Grafica(JPanel Panel, double plotWindowTime){
	   grafico = new Grafic("Grafico",Panel,plotWindowTime) ;
   }
   
   /**
    * Creacion del menu Principal
    * @param cl	Listener para el Menu de Configuracion
    * @param wl Listener para el Menu MyWorld 
    * @param world Instancia de MyWorld
    * @return JMenuBar MenuBar del menu Principal de la aplicacion
    */
   public JMenuBar createLabMenuBar(ConfigurationMenuListener cl, WorldControlMenuListener wl,MyWorld world) {
	      JMenuBar mb = new JMenuBar();
	      
	      JMenu menu = new JMenu ("Configuration");
	      mb.add(menu);
	      JMenu subMenu = new JMenu("Insert");  
	      menu.add(subMenu);
	    //aniadimos opcion "Delete" al menu de Configuration, junto con el listener
	      JMenuItem menuItem = new JMenuItem("Delete"); 
	      menuItem.addActionListener(cl);
	      menu.add(menuItem);
	    //aniadimos opcion "Edit Elements" al menu de Configuration, junto con el listener
	      menuItem = new JMenuItem("Edit Elements");  
	      menuItem.addActionListener(cl);
	      menu.add(menuItem);
	      
	      menuItem = new JMenuItem("Ball");
	      menuItem.addActionListener(cl);
	      subMenu.add(menuItem);
	      menuItem = new JMenuItem("Fixed Hook");
	      menuItem.addActionListener(cl);
	      subMenu.add(menuItem);
	      menuItem = new JMenuItem("Spring");
	      menuItem.addActionListener(cl);
	      subMenu.add(menuItem);
	     //aniadimos opcion "Elastic" al submenu de Insert de Configuration, junto con el listener
	      menuItem = new JMenuItem("Elastic");
	      menuItem.addActionListener(cl);
	      subMenu.add(menuItem);
	      
	      menu = new JMenu("MyWorld");
	      mb.add(menu);
	      menuItem = new JMenuItem("Start");
	      menuItem.addActionListener(wl);
	      menu.add(menuItem);
	      //aniadimos opcion "Resumen" al menu de MyWorld, junto con el listener
	      menuItem = new JMenuItem("Resume");
	      menuItem.addActionListener(wl);
	      menu.add(menuItem);
	      menuItem = new JMenuItem("Stop");
	      menuItem.addActionListener(wl);
	      menu.add(menuItem);
	      
	      menuItem = new JMenuItem("Gravity");
	      menuItem.addActionListener(wl);
	      menu.add(menuItem);
	      menuItem = new JMenuItem("Viscosity");
	      menuItem.addActionListener(wl);
	      menu.add(menuItem);
	      subMenu = new JMenu("Simulator");
	      menu.add(subMenu);
	      menuItem = new JMenuItem("Delta time");
	      menuItem.addActionListener(wl);
	      subMenu.add(menuItem);
	      menuItem = new JMenuItem("Graphics Refresh time");
	      menuItem.addActionListener(wl);
	      subMenu.add(menuItem);
	      mb.add(menu);
	      mb.add(world.CreaMenu());
	      return mb;
	 } 
   
} 
