import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import javax.swing.*;


public class PhysicsLab {
   public static void main(String[] args) {
      PhysicsLabFrame labFrame = new PhysicsLabFrame();
      labFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      labFrame.setVisible(true);
   }
}

class PhysicsLabFrame extends JFrame {
/**
 * Crea un nuevo Frame para PhysicsLab version aplicacion
 * Se obtienen los valores de configuracion de la simulacion
 * provenientes del archivo PhysicsLab.ini
 */
public PhysicsLabFrame() {
	
      setTitle("Small Physics Laboratory");
      setSize(MyWorld.WIDTH, MyWorld.HEIGHT+50);
      
      
      
      Properties prop = new Properties();
      try{
      FileInputStream fichero = new FileInputStream("PhysicsLab.ini");
      prop.load(fichero);
      }catch(IOException e){
    	  	System.err.println("No existe archivo PhysicsLab.ini");
    	  	}
      
      //En caso que no existan los parametros toman el valor por defecto
      //el cual es el 2do argumento de getPropierty
      String strRefreshTime = prop.getProperty("RefreshTime","0.06");
      String strDeltaTime = prop.getProperty("DeltaTime","0.0001");
      String strPlotWindowTime = prop.getProperty("PlotWindowTime","30");
      
      //En caso de que existe la clave en el archivo .ini pero
      //sin un valor, toma el valor por defecto
      if(strRefreshTime.equals(""))
    	  strRefreshTime = "0.06";
      if(strDeltaTime.equals(""))
    	  strDeltaTime = "0.0001";
      if(strPlotWindowTime.equals(""))
    	  strPlotWindowTime = "30";
      double refreshTime= Double.parseDouble(strRefreshTime);
      double deltaTime = Double.parseDouble(strDeltaTime);
      double plotWindowTime = Double.parseDouble(strPlotWindowTime);
      
      MyWorld world = new MyWorld(refreshTime,deltaTime);
      
	     
      PhysicsLabPanel labPanel = new PhysicsLabPanel(world);
      JPanel fondo = new JPanel();
     
      world.Grafica(fondo,plotWindowTime);
      JSplitPane panelPrincipal = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,labPanel,fondo);
      panelPrincipal.setDividerLocation(MyWorld.WIDTH/2);
      
      add(panelPrincipal);  // it is equivalent to getContentPane().add(...);
      world.setPanel(labPanel);   // to repaint view when model changes
      ConfigurationMenuListener cListener = new ConfigurationMenuListener(world,labPanel);
      WorldControlMenuListener wcListener = new WorldControlMenuListener(world);
      setJMenuBar(world.createLabMenuBar(cListener, wcListener,world));
   }

   public JMenuBar createLabMenuBar(ConfigurationMenuListener cl, WorldControlMenuListener wl) {
      JMenuBar mb = new JMenuBar();
      
      JMenu menu = new JMenu ("Configuration");
      mb.add(menu);
      JMenu subMenu = new JMenu("Insert");  
      menu.add(subMenu);
    //aniadimos opcion "Delete" al menu de Configuration, junto con el listener
      JMenuItem menuItem = new JMenuItem("Delete"); 
      menuItem.addActionListener(cl);
      menu.add(menuItem);
    //aniadimos opcion "Edit Elements" al menu de Configuration, junto con el listener
      menuItem = new JMenuItem("Edit Elements");  
      menuItem.addActionListener(cl);
      menu.add(menuItem);
      
      menuItem = new JMenuItem("Ball");
      menuItem.addActionListener(cl);
      subMenu.add(menuItem);
      menuItem = new JMenuItem("Fixed Hook");
      menuItem.addActionListener(cl);
      subMenu.add(menuItem);
      menuItem = new JMenuItem("Spring");
      menuItem.addActionListener(cl);
      subMenu.add(menuItem);
     //aniadimos opcion "Elastic" al submenu de Insert de Configuration, junto con el listener
      menuItem = new JMenuItem("Elastic");
      menuItem.addActionListener(cl);
      subMenu.add(menuItem);
       
      menu = new JMenu("MyWorld");
      mb.add(menu);
      menuItem = new JMenuItem("Start");
      menuItem.addActionListener(wl);
      menu.add(menuItem);
      //aniadimos opcion "Resumen" al menu de MyWorld, junto con el listener
      menuItem = new JMenuItem("Resume");
      menuItem.addActionListener(wl);
      menu.add(menuItem);
      menuItem = new JMenuItem("Stop");
      menuItem.addActionListener(wl);
      menu.add(menuItem);
      
      menuItem = new JMenuItem("Gravity");
      menuItem.addActionListener(wl);
      menu.add(menuItem);
      menuItem = new JMenuItem("Viscosity");
      menuItem.addActionListener(wl);
      menu.add(menuItem);
      subMenu = new JMenu("Simulator");
      menu.add(subMenu);
      menuItem = new JMenuItem("Delta time");
      menuItem.addActionListener(wl);
      subMenu.add(menuItem);
      menuItem = new JMenuItem("Graphics Refresh time");
      menuItem.addActionListener(wl);
      subMenu.add(menuItem);
      
      //Para seleccionar las masas a graficar.
      menu = new JMenu("Select");
      menu.addActionListener(wl);
      mb.add(menu);
      
      return mb;          
   }   
}
