
import javax.swing.*;

/**
 * Clase para crear el Applet
 *
 */

public class PhysicsLabApplet extends JApplet{
	 /**
	 * Applet de PhysicsLab
	 * Se usa getParameter para obtener los valores desde el html
	 * init() es basicamente el constructor de PhysicsLabFrame, salvo algunas 
	 * limitaciones propias de un applet, como lo es el tamanio de la ventana 
	 * y la forma de obtener parametros de entrada (vs .ini de la version aplicacion)
	 */


	public void init()
	   { 
	      
	      double refreshTime = 0;
	      double deltaTime = 0;
	      double plotWindowTime = 0;
	      
	      String strPlotWindowTime = getParameter("PlotWindowTime");
	      String strDeltaTime = getParameter("DeltaTime");
	      String strRefreshTime = getParameter("RefreshTime");
	      if(strPlotWindowTime == null)
	    	  plotWindowTime = 30; //valor por defecto en caso de no estar especificado en el html
	      else 
	    	  plotWindowTime = Double.parseDouble(strPlotWindowTime);
	      
	      
	      if(strDeltaTime == null)
	    	  deltaTime = 0.0001; //valor por defecto en caso de no estar especificado en el html
	      else 
	    	  deltaTime = Double.parseDouble(strDeltaTime);
	      
	     
	      if(strRefreshTime == null)
	    	  refreshTime = 0.06; //valor por defecto en caso de no estar especificado en el html
	      else 
	    	  refreshTime = Double.parseDouble(strRefreshTime);
	      
	      System.out.println(plotWindowTime + " " + refreshTime + " " + deltaTime);
	     
	      MyWorld world = new MyWorld(refreshTime,deltaTime);
	      PhysicsLabPanel labPanel = new PhysicsLabPanel(world);
	      JPanel fondo = new JPanel();
	     
          world.Grafica(fondo,plotWindowTime);
	      JSplitPane panelPrincipal = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,labPanel,fondo);
	      panelPrincipal.setDividerLocation(MyWorld.WIDTH/2);
	      
	      add(panelPrincipal);  // it is equivalent to getContentPane().add(...);
	      world.setPanel(labPanel);   // to repaint view when model changes
	       ConfigurationMenuListener cListener = new ConfigurationMenuListener(world,labPanel);
	      WorldControlMenuListener wcListener = new WorldControlMenuListener(world);
	      setJMenuBar(world.createLabMenuBar(cListener, wcListener,world));
	      
		   }
	 
	   /**
    * Creacion del menu Principal
    * @param cl	Listener para el Menu de Configuracion
    * @param wl Listener para el Menu MyWorld 
    * @return JMenuBar MenuBar del menu Principal de la aplicacion
    */
	 public JMenuBar createLabMenuBar(ConfigurationMenuListener cl, WorldControlMenuListener wl) {
	      JMenuBar mb = new JMenuBar();
	      
	      JMenu menu = new JMenu ("Configuration");
	      mb.add(menu);
	      JMenu subMenu = new JMenu("Insert");  
	      menu.add(subMenu);
	    //aniadimos opcion "Delete" al menu de Configuration, junto con el listener
	      JMenuItem menuItem = new JMenuItem("Delete"); 
	      menuItem.addActionListener(cl);
	      menu.add(menuItem);
	    //aniadimos opcion "Edit Elements" al menu de Configuration, junto con el listener
	      menuItem = new JMenuItem("Edit Elements");  
	      menuItem.addActionListener(cl);
	      menu.add(menuItem);
	      
	      menuItem = new JMenuItem("Ball");
	      menuItem.addActionListener(cl);
	      subMenu.add(menuItem);
	      menuItem = new JMenuItem("Fixed Hook");
	      menuItem.addActionListener(cl);
	      subMenu.add(menuItem);
	      menuItem = new JMenuItem("Spring");
	      menuItem.addActionListener(cl);
	      subMenu.add(menuItem);
	     //aniadimos opcion "Elastic" al submenu de Insert de Configuration, junto con el listener
	      menuItem = new JMenuItem("Elastic");
	      menuItem.addActionListener(cl);
	      subMenu.add(menuItem);
	      
	      
	      menu = new JMenu("MyWorld");
	      mb.add(menu);
	      menuItem = new JMenuItem("Start");
	      menuItem.addActionListener(wl);
	      menu.add(menuItem);
	      //aniadimos opcion "Resumen" al menu de MyWorld, junto con el listener
	      menuItem = new JMenuItem("Resume");
	      menuItem.addActionListener(wl);
	      menu.add(menuItem);
	      menuItem = new JMenuItem("Stop");
	      menuItem.addActionListener(wl);
	      menu.add(menuItem);
	      
	      menuItem = new JMenuItem("Gravity");
	      menuItem.addActionListener(wl);
	      menu.add(menuItem);
	      menuItem = new JMenuItem("Viscosity");
	      menuItem.addActionListener(wl);
	      menu.add(menuItem);
	      subMenu = new JMenu("Simulator");
	      menu.add(subMenu);
	      menuItem = new JMenuItem("Delta time");
	      menuItem.addActionListener(wl);
	      subMenu.add(menuItem);
	      menuItem = new JMenuItem("Graphics Refresh time");
	      menuItem.addActionListener(wl);
	      subMenu.add(menuItem);
	      menu = new JMenu("Select");
	      mb.add(menu);
	      
	      return mb;          
	   }   

	 
	 
	 

}
